declare module 'ember' {
  /**
    @module ember
    */
  import require from 'require';
  import * as utils from '@ember/-internals/utils';
  import { Registry, Container } from '@ember/-internals/container';
  import * as instrumentation from '@ember/instrumentation';
  import * as metal from '@ember/-internals/metal';
  import { isEnabled } from '@ember/canary-features';
  import * as EmberDebug from '@ember/debug';
  import Backburner from 'backburner.js';
  import Controller, { inject as injectController } from '@ember/controller';
  import Service from '@ember/service';
  import EmberObject, { action, observer } from '@ember/object';
  import { dependentKeyCompat } from '@ember/object/compat';
  import { RSVP } from '@ember/-internals/runtime';
  import {
    setComponentManager,
    escapeExpression,
    template,
    isSerializationFirstNode,
    type TemplatesRegistry,
  } from '@ember/-internals/glimmer';
  import * as views from '@ember/-internals/views';
  import ContainerDebugAdapter from '@ember/debug/container-debug-adapter';
  import DataAdapter from '@ember/debug/data-adapter';
  import { run } from '@ember/runloop';
  import { NativeArray, isArray, makeArray } from '@ember/array';
  import ArrayProxy from '@ember/array/proxy';
  import Application, { onLoad, runLoadHooks } from '@ember/application';
  import ApplicationInstance from '@ember/application/instance';
  import Namespace from '@ember/application/namespace';
  import Component from '@ember/component';
  import Helper from '@ember/component/helper';
  import Engine from '@ember/engine';
  import EngineInstance from '@ember/engine/instance';
  import CoreObject from '@ember/object/core';
  import Mixin, { mixin } from '@ember/object/mixin';
  import ObjectProxy from '@ember/object/proxy';
  import HashLocation from '@ember/routing/hash-location';
  import HistoryLocation from '@ember/routing/history-location';
  import NoneLocation from '@ember/routing/none-location';
  import Route from '@ember/routing/route';
  import Router from '@ember/routing/router';
  import {
    controllerFor,
    generateController,
    generateControllerFactory,
    DSL as RouterDSL,
  } from '@ember/routing/-internals';
  import { isNone, isBlank, isEmpty, isPresent, isEqual, typeOf, compare } from '@ember/utils';
  import * as glimmerRuntime from '@glimmer/runtime';
  import {
    helperCapabilities,
    setModifierManager,
    setComponentTemplate,
    getComponentTemplate,
    setHelperManager,
  } from '@glimmer/manager';
  import {
    associateDestroyableChild,
    registerDestructor,
    unregisterDestructor,
  } from '@ember/destroyable';
  import type { precompile, compile } from 'ember-template-compiler';
  import type * as EmberTesting from 'ember-testing';
  /**
      Namespace for injection helper methods.

      @class inject
      @namespace Ember
      @static
      @public
    */
  function inject(): void;
  const PartialEmber: {
    readonly isNamespace: true;
    readonly toString: () => string;
    readonly Container: typeof Container;
    readonly Registry: typeof Registry;
    readonly _setComponentManager: typeof setComponentManager;
    readonly _componentManagerCapabilities: typeof import('@glimmer/manager').componentCapabilities;
    readonly _modifierManagerCapabilities: typeof import('@glimmer/manager').modifierCapabilities;
    readonly meta: {
      (obj: object): import('@ember/-internals/meta').Meta;
      _counters?: import('@ember/-internals/meta').MetaCounters | undefined;
    };
    readonly _createCache: typeof metal.createCache;
    readonly _cacheGetValue: typeof metal.getValue;
    readonly _cacheIsConst: typeof metal.isConst;
    readonly _descriptor: typeof metal.nativeDescDecorator;
    readonly _getPath: typeof metal._getPath;
    readonly _setClassicDecorator: typeof metal.setClassicDecorator;
    readonly _tracked: typeof metal.tracked;
    readonly beginPropertyChanges: typeof metal.beginPropertyChanges;
    readonly changeProperties: typeof metal.changeProperties;
    readonly endPropertyChanges: typeof metal.endPropertyChanges;
    readonly hasListeners: typeof metal.hasListeners;
    readonly libraries: metal.Libraries;
    readonly _ContainerProxyMixin: Mixin;
    readonly _ProxyMixin: Mixin;
    readonly _RegistryProxyMixin: Mixin;
    readonly ActionHandler: Mixin;
    readonly Comparable: Mixin;
    readonly RSVP: typeof RSVP;
    readonly ComponentLookup: Readonly<typeof EmberObject> &
      (new (owner?: import('@ember/owner').default | undefined) => EmberObject);
    readonly EventDispatcher: typeof views.EventDispatcher;
    readonly _Cache: typeof utils.Cache;
    readonly GUID_KEY: `__ember${number}`;
    readonly canInvoke: typeof utils.canInvoke;
    readonly generateGuid: typeof utils.generateGuid;
    readonly guidFor: typeof utils.guidFor;
    readonly uuid: typeof utils.uuid;
    readonly wrap: typeof utils.wrap;
    readonly getOwner: (object: object) => import('@ember/owner').default | undefined;
    readonly onLoad: typeof onLoad;
    readonly runLoadHooks: typeof runLoadHooks;
    readonly setOwner: typeof import('@ember/owner').setOwner;
    readonly Application: typeof Application;
    readonly ApplicationInstance: typeof ApplicationInstance;
    readonly Namespace: typeof Namespace;
    readonly A: <T>(arr?: T[] | undefined) => NativeArray<T>;
    readonly Array: Mixin;
    readonly NativeArray: Mixin;
    readonly isArray: typeof isArray;
    readonly makeArray: typeof makeArray;
    readonly MutableArray: Mixin;
    readonly ArrayProxy: typeof ArrayProxy;
    readonly FEATURES: {
      readonly isEnabled: typeof isEnabled;
    };
    readonly _Input: import('@ember/-internals/glimmer/lib/components/internal').OpaqueInternalComponentConstructor;
    readonly Component: typeof Component;
    readonly Helper: typeof Helper;
    readonly Controller: typeof Controller;
    readonly ControllerMixin: Mixin;
    readonly _captureRenderTree: typeof EmberDebug.captureRenderTree;
    readonly assert: EmberDebug.AssertFunc;
    readonly warn: import('@ember/debug/lib/warn').WarnFunc;
    readonly debug: EmberDebug.DebugFunc;
    readonly deprecate: import('@ember/debug/lib/deprecate').DeprecateFunc;
    readonly deprecateFunc: EmberDebug.DeprecateFuncFunc;
    readonly runInDebug: EmberDebug.RunInDebugFunc;
    readonly inspect: typeof EmberDebug.inspect;
    readonly Debug: {
      readonly registerDeprecationHandler: (
        handler: import('@ember/debug/lib/handlers').HandlerCallback<EmberDebug.DeprecationOptions>
      ) => void;
      readonly registerWarnHandler: import('@ember/debug/lib/warn').RegisterHandlerFunc;
      readonly isComputed: typeof metal.isComputed;
    };
    readonly ContainerDebugAdapter: typeof ContainerDebugAdapter;
    readonly DataAdapter: typeof DataAdapter;
    readonly _assertDestroyablesDestroyed: (() => void) | undefined;
    readonly _associateDestroyableChild: typeof associateDestroyableChild;
    readonly _enableDestroyableTracking: (() => void) | undefined;
    readonly _isDestroying: typeof glimmerRuntime.isDestroying;
    readonly _isDestroyed: typeof glimmerRuntime.isDestroyed;
    readonly _registerDestructor: typeof registerDestructor;
    readonly _unregisterDestructor: typeof unregisterDestructor;
    readonly destroy: typeof glimmerRuntime.destroy;
    readonly Engine: typeof Engine;
    readonly EngineInstance: typeof EngineInstance;
    readonly Enumerable: Mixin;
    readonly MutableEnumerable: Mixin;
    readonly instrument: typeof instrumentation.instrument;
    readonly subscribe: typeof instrumentation.subscribe;
    readonly Instrumentation: {
      readonly instrument: typeof instrumentation.instrument;
      readonly subscribe: typeof instrumentation.subscribe;
      readonly unsubscribe: typeof instrumentation.unsubscribe;
      readonly reset: typeof instrumentation.reset;
    };
    readonly Object: typeof EmberObject;
    readonly _action: typeof action;
    readonly computed: typeof metal.computed;
    readonly defineProperty: typeof metal.defineProperty;
    readonly get: typeof metal.get;
    readonly getProperties: typeof metal.getProperties;
    readonly notifyPropertyChange: typeof metal.notifyPropertyChange;
    readonly observer: typeof observer;
    readonly set: typeof metal.set;
    readonly trySet: typeof metal.trySet;
    readonly setProperties: typeof metal.setProperties;
    readonly cacheFor: typeof metal.getCachedValueFor;
    readonly _dependentKeyCompat: typeof dependentKeyCompat;
    readonly ComputedProperty: typeof metal.ComputedProperty;
    readonly expandProperties: typeof metal.expandProperties;
    readonly CoreObject: typeof CoreObject;
    readonly Evented: Mixin;
    readonly on: typeof metal.on;
    readonly addListener: typeof metal.addListener;
    readonly removeListener: typeof metal.removeListener;
    readonly sendEvent: typeof metal.sendEvent;
    readonly Mixin: typeof Mixin;
    readonly mixin: typeof mixin;
    readonly Observable: Mixin;
    readonly addObserver: typeof metal.addObserver;
    readonly removeObserver: typeof metal.removeObserver;
    readonly PromiseProxyMixin: Mixin;
    readonly ObjectProxy: typeof ObjectProxy;
    readonly RouterDSL: typeof RouterDSL;
    readonly controllerFor: typeof controllerFor;
    readonly generateController: typeof generateController;
    readonly generateControllerFactory: typeof generateControllerFactory;
    readonly HashLocation: typeof HashLocation;
    readonly HistoryLocation: typeof HistoryLocation;
    readonly NoneLocation: typeof NoneLocation;
    readonly Route: typeof Route;
    readonly Router: typeof Router;
    readonly run: typeof run;
    readonly Service: typeof Service;
    readonly compare: typeof compare;
    readonly isBlank: typeof isBlank;
    readonly isEmpty: typeof isEmpty;
    readonly isEqual: typeof isEqual;
    readonly isNone: typeof isNone;
    readonly isPresent: typeof isPresent;
    readonly typeOf: typeof typeOf;
    /**
          The semantic version
      
          @property VERSION
          @type String
          @public
        */
    readonly VERSION: string;
    readonly ViewUtils: {
      readonly getChildViews: typeof views.getChildViews;
      readonly getElementView: typeof views.getElementView;
      readonly getRootViews: typeof views.getRootViews;
      readonly getViewBounds: typeof views.getViewBounds;
      readonly getViewBoundingClientRect: typeof views.getViewBoundingClientRect;
      readonly getViewClientRects: typeof views.getViewClientRects;
      readonly getViewElement: typeof views.getViewElement;
      readonly isSimpleClick: typeof views.isSimpleClick;
      readonly isSerializationFirstNode: typeof isSerializationFirstNode;
    };
    readonly _getComponentTemplate: typeof getComponentTemplate;
    readonly _helperManagerCapabilities: typeof helperCapabilities;
    readonly _setComponentTemplate: typeof setComponentTemplate;
    readonly _setHelperManager: typeof setHelperManager;
    readonly _setModifierManager: typeof setModifierManager;
    readonly _templateOnlyComponent: typeof glimmerRuntime.templateOnlyComponent;
    readonly _invokeHelper: typeof glimmerRuntime.invokeHelper;
    readonly _hash: object;
    readonly _array: object;
    readonly _concat: object;
    readonly _get: object;
    readonly _on: {};
    readonly _fn: object;
    readonly _Backburner: typeof Backburner;
    readonly inject: typeof inject;
    readonly __loader: {
      readonly require: typeof require;
      readonly define: typeof define;
      readonly registry: any;
    };
  };
  interface EmberHandlebars {
    template: typeof template;
    Utils: {
      escapeExpression: typeof escapeExpression;
    };
    compile?: typeof compile;
    precompile?: typeof precompile;
  }
  interface EmberHTMLBars {
    template: typeof template;
    compile?: typeof compile;
    precompile?: typeof precompile;
  }
  type PartialEmber = typeof PartialEmber;
  interface Ember extends PartialEmber {
    get ENV(): object;
    get lookup(): Record<string, unknown>;
    set lookup(value: Record<string, unknown>);
    /**
          A function may be assigned to `Ember.onerror` to be called when Ember
          internals encounter an error. This is useful for specialized error handling
          and reporting code.
      
          ```javascript
      
          Ember.onerror = function(error) {
            const payload = {
              stack: error.stack,
              otherInformation: 'whatever app state you want to provide'
            };
      
            fetch('/report-error', {
              method: 'POST',
              body: JSON.stringify(payload)
            });
          };
          ```
      
          Internally, `Ember.onerror` is used as Backburner's error handler.
      
          @event onerror
          @for Ember
          @param {Exception} error the error object
          @public
        */
    get onerror(): Function | undefined;
    set onerror(handler: Function | undefined);
    get testing(): boolean;
    set testing(value: boolean);
    /**
          Whether searching on the global for new Namespace instances is enabled.
      
          This is only exported here as to not break any addons.  Given the new
          visit API, you will have issues if you treat this as a indicator of
          booted.
      
          Internally this is only exposing a flag in Namespace.
      
          @property BOOTED
          @for Ember
          @type Boolean
          @private
        */
    get BOOTED(): boolean;
    set BOOTED(flag: boolean);
    /**
          Global hash of shared templates. This will automatically be populated
          by the build tools so that you can store your Handlebars templates in
          separate files that get loaded into JavaScript at buildtime.
      
          @property TEMPLATES
          @for Ember
          @type Object
          @private
        */
    get TEMPLATES(): TemplatesRegistry;
    set TEMPLATES(registry: TemplatesRegistry);
    HTMLBars: EmberHTMLBars;
    Handlebars: EmberHandlebars;
    Test?: typeof EmberTesting['Test'] & {
      Adapter: typeof EmberTesting['Adapter'];
      QUnitAdapter: typeof EmberTesting['QUnitAdapter'];
    };
    setupForTesting?: typeof EmberTesting['setupForTesting'];
  }
  const Ember: Ember;
  let EmberHandlebars: EmberHandlebars;
  let EmberHTMLBars: EmberHTMLBars;
  export default Ember;
}

declare namespace inject {
  var controller: typeof injectController;
  var service: typeof import('@ember/service').service;
}
